import os

import requests
from howlongtobeatpy import HowLongToBeat

from playa.models import Game


class HLTBClient:
    def search_game(self, title) -> Game | None:
        try:
            results = HowLongToBeat().search(title)
            if results is not None and len(results) > 0:
                # Return the best match
                return max(results, key=lambda element: element.similarity)
        except Exception:
            return None
        return None


class IGDBClient:
    def __init__(self) -> None:
        self.client_id = os.getenv('TWITCH_CLIENT_ID')
        self.client_secret = os.getenv('TWITCH_CLIENT_SECRET')
        self.access_token = None
        self.base_url = 'https://api.igdb.com/v4'

    def get_access_token(self) -> None:
        url = 'https://id.twitch.tv/oauth2/token'
        data = {
            'client_id': self.client_id,
            'client_secret': self.client_secret,
            'grant_type': 'client_credentials',
        }
        response = requests.post(url, data=data)
        self.access_token = response.json()['access_token']

    def search_games(self, query):
        if not self.access_token:
            self.get_access_token()

        headers = {
            'Client-ID': self.client_id,
            'Authorization': f'Bearer {self.access_token}',
        }

        body = f"""
            search "{query}";
            fields name, id, first_release_date, platforms.name, cover.url, 
                   game_modes.name;
            limit 10;
        """

        response = requests.post(f'{self.base_url}/games', headers=headers, data=body)
        return response.json()

    def get_game_details(self, game_id):
        if not self.access_token:
            self.get_access_token()

        headers = {
            'Client-ID': self.client_id,
            'Authorization': f'Bearer {self.access_token}',
        }

        body = f"""
            where id = {game_id};
            fields name, first_release_date, platforms.name, cover.url,
                   game_modes.name;
        """

        response = requests.post(f'{self.base_url}/games', headers=headers, data=body)
        return response.json()[0]

    def get_time_to_beat(self, game_id):
        if not self.access_token:
            self.get_access_token()

        headers = {
            'Client-ID': self.client_id,
            'Authorization': f'Bearer {self.access_token}',
        }

        body = f"""
            where id = {game_id};
            fields completely, hastily, normally;
        """

        response = requests.post(f'{self.base_url}/game_time_to_beats', headers=headers, data=body)
        ttb = response.json()

        if ttb and len(ttb) > 0:
            return ttb[0]
        else:
            return None
