import { Controller } from "https://unpkg.com/@hotwired/stimulus@3.2.2/dist/stimulus.js"

export default class extends Controller {
    static targets = ["list", "detail", "status", "platforms", "progress", "progressBar", "progressLabel", "rating", "ratingStars", "notes", "message"]
    static values = {
        updateUrl: String
    }

    showList() {
        this.listTarget.classList.remove("hidden")
        this.detailTarget.classList.add("hidden")
    }

    showDetail() {
        this.listTarget.classList.add("hidden")
        this.detailTarget.classList.remove("hidden")
    }

    async updateField(event) {
        const target = event.currentTarget;
        const field = target.getAttribute('name');
        const value = target.getAttribute('data-value') || target.value;

        try {
            const response = await fetch(this.updateUrlValue, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    'X-CSRFToken': document.querySelector('[name=csrfmiddlewaretoken]').value
                },
                body: JSON.stringify({
                    field: field,
                    value: value
                })
            });

            const data = await response.json();
            if (data.status === 'success') {
                this.showMessage('Updated successfully', 'success');
            } else {
                this.showMessage('Update failed', 'error');
            }
        } catch (error) {
            this.showMessage('Update failed', 'error');
        }
    }

    async saveNotes(event) {
        event.preventDefault();
        const notes = this.notesTarget.value;
        
        try {
            const response = await fetch(this.updateUrlValue, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    'X-CSRFToken': document.querySelector('[name=csrfmiddlewaretoken]').value
                },
                body: JSON.stringify({
                    field: 'notes',
                    value: notes
                })
            });

            const data = await response.json();
            if (data.status === 'success') {
                this.showMessage('Notes saved successfully', 'success');
            } else {
                this.showMessage('Failed to save notes', 'error');
            }
        } catch (error) {
            this.showMessage('Failed to save notes', 'error');
        }
    }

    handleStarHover(event) {
        const stars = this.ratingStarsTarget.querySelectorAll('.star');
        const hoveredStar = event.currentTarget;
        const hoverValue = parseInt(hoveredStar.dataset.value);
        const currentRating = parseInt(this.ratingTarget.value) || 0;
        
        stars.forEach(star => {
            const starValue = parseInt(star.dataset.value);
            star.classList.remove('text-yellow-300', 'text-yellow-200', 'text-gray-300', 'dark:text-gray-600');
            
            if (starValue <= hoverValue) {
                star.classList.add('text-yellow-200');
            } else {
                star.classList.add('text-gray-300', 'dark:text-gray-600');
            }
        });
    }

    handleStarLeave() {
        const stars = this.ratingStarsTarget.querySelectorAll('.star');
        const currentRating = parseInt(this.ratingTarget.value) || 0;
        
        stars.forEach(star => {
            const starValue = parseInt(star.dataset.value);
            star.classList.remove('text-yellow-300', 'text-yellow-200', 'text-gray-300', 'dark:text-gray-600');
            
            if (starValue <= currentRating) {
                star.classList.add('text-yellow-300');
            } else {
                star.classList.add('text-gray-300', 'dark:text-gray-600');
            }
        });
    }

    async handleStarClick(event) {
        const star = event.currentTarget;
        const value = star.dataset.value;
        this.ratingTarget.value = value;
        await this.updateField({
            currentTarget: {
                getAttribute: (name) => name === 'name' ? 'rating' : value,
                value: value
            }
        });
        this.updateStarDisplay(value);
    }

    updateStarDisplay(rating) {
        const stars = this.ratingStarsTarget.querySelectorAll('.star');
        const ratingValue = parseInt(rating) || 0;
        
        stars.forEach(star => {
            const starValue = parseInt(star.dataset.value);
            star.classList.remove('text-yellow-300', 'text-yellow-200', 'text-gray-300', 'dark:text-gray-600');
            if (starValue <= ratingValue) {
                star.classList.add('text-yellow-300');
            } else {
                star.classList.add('text-gray-300', 'dark:text-gray-600');
            }
        });
    }

    updateProgress(event) {
        const value = event.currentTarget.value;
        this.progressBarTarget.style.width = `${value}%`;
        this.progressLabelTarget.textContent = `${value}%`;
    }

    async updateField(event) {
        const target = event.currentTarget;
        const field = target.getAttribute('name');
        const value = target.getAttribute('data-value') || target.value;

        try {
            const response = await fetch(this.updateUrlValue, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    'X-CSRFToken': document.querySelector('[name=csrfmiddlewaretoken]').value
                },
                body: JSON.stringify({
                    field: field,
                    value: value
                })
            });

            const data = await response.json();
            if (data.status === 'success') {
                this.showMessage('Updated successfully', 'success');
                if (field === 'progress') {
                    this.progressBarTarget.style.width = `${value}%`;
                    this.progressLabelTarget.textContent = `${value}%`;
                }
            } else {
                this.showMessage('Update failed', 'error');
            }
        } catch (error) {
            this.showMessage('Update failed', 'error');
        }
    }

    showMessage(text, type) {
        this.messageTarget.textContent = text;
        this.messageTarget.classList.remove('hidden', 'text-red-600', 'text-green-600');
        this.messageTarget.classList.add(type === 'success' ? 'text-green-600' : 'text-red-600');
        
        setTimeout(() => {
            this.messageTarget.classList.add('hidden');
        }, 3000);
    }
}
